package net.sf.sidaof.dao.hibernate;

import net.sf.sidaof.dao.Dao;

import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;

/**
 * Base interface for Hibernate DAOs. To use, extend it in DAO interfaces.
 * 
 * @param <K>
 *            The primary key class of the entity, e.g. Long.
 * @param <E>
 *            The entity class the DAO persists, e.g. Person.
 */
public interface HibernateDao<K, E> extends Dao<K, E> {

    /**
     * Find an entity by its id, optionally throwing an exception if not found
     * (this prevents <code>NullPointerException</code>s from occurring later),
     * and optionally locking the found entity.
     * 
     * @param id
     *            The id (primary key) of the entity to find.
     * @param lockMode
     *            Lock the entity in the persistence context for the specified
     *            lockMode, or null for no locking.
     * @param exceptionIfNotFound
     *            true to throw an exception if entity not found.
     * @return The entity for the specified id, null when entity not found.
     */
    E findById(K id, LockMode lockMode, boolean exceptionIfNotFound);

    /**
     * Run the specified query, using getSingleResult(), expecting a single
     * entity result and returning null if not found.
     * 
     * @param q
     *            The query to run getSingleResult() on.
     * @return The single entity found from the query execution or null if not
     *         found.
     */
    E runQueryWithSingleResult(Query q);

    /**
     * Run the specified query, using getSingleResult(), expecting a single
     * entity result and returning null if not found.
     * 
     * @param q
     *            The query to run getSingleResult() on.
     * @param lockMode
     *            Lock the entity in the persistence context for the specified
     *            lockMode, or null for no locking.
     * @param exceptionIfNotFound
     *            true to throw an exception if entity not found.
     * @return The single entity found from the query execution.
     */
    E runQueryWithSingleResult(Query q, LockMode lockMode,
            boolean exceptionIfNotFound);

    /**
     * Get the Hibernate SessionFactory used for operations.
     * 
     * @return The Hibernate SessionFactory used for operations.
     */
    SessionFactory getSessionFactory();

    /**
     * Set the Hibernate SessionFactory used for operations.
     * 
     * @param session
     *            The Hibernate SessionFactory used for operations.
     */
    void setSessionFactory(SessionFactory sessionFactory);

    /**
     * Get the Hibernate Session used for operations.
     * 
     * @return The Hibernate Session used for operations.
     */
    Session getSession();
}
