package net.sf.sidaof.spring;

import java.util.Collection;
import java.util.List;

import net.sf.sidaof.dao.Dao;

import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * This class provides a Spring transaction boundary for DAO methods, executing
 * each method in a new transaction. It provides an easy way to "wrap" an
 * existing DAO's method calls in a new transaction.
 * </p>
 * <p>
 * For example, this approach is needed for tests that insert database prep data
 * (before testing the "production" persist/update/delete method) or read the
 * results of database operations (after testing the "production"
 * persist/update/delete method) differently than the production approach, e.g.
 * JDBC with auto-commit vs JPA entity manager, in a non-JTA environment.
 * 
 * All methods run in a new transaction so the code under test and data
 * validation will see the persisted data. Without this transaction boundary,
 * persist tests fail due to deadlocks in a non-JTA environment.
 * </p>
 * <p>
 * To use:
 * </p>
 * <ul>
 * <li>Extend and implement the <code>getDao()</code> method.</li>
 * <li>Add @Repository (or other way of instantiating DAO) to the concrete
 * class.</li>
 * <li>If not using Spring transactions, then also add way of starting new
 * transaction for each method.</li>
 * <li>Override methods as necessary to add additional processing in the new
 * transaction.</li>
 * </ul>
 * </p>
 * 
 * @author Jeff Jensen
 */
@Transactional(propagation = Propagation.REQUIRES_NEW)
public abstract class TransactionBoundaryDao<K, E> {
    /**
     * @see net.sf.sidaof.dao.Dao#clear()
     */
    public void clear() {
        getDao().clear();
    }

    /**
     * @param entity
     * @return
     * @see net.sf.sidaof.dao.Dao#contains(java.lang.Object)
     */
    public boolean contains(E entity) {
        return getDao().contains(entity);
    }

    /**
     * @return
     * @see net.sf.sidaof.dao.Dao#countAll()
     */
    public Long countAll() {
        return getDao().countAll();
    }

    /**
     * @param where
     * @param pathExpression
     * @return
     * @see net.sf.sidaof.dao.Dao#countAll(java.lang.String, java.lang.String)
     */
    public Long countAll(String where, String pathExpression) {
        return getDao().countAll(where, pathExpression);
    }

    /**
     * @param where
     * @return
     * @see net.sf.sidaof.dao.Dao#countAll(java.lang.String)
     */
    public Long countAll(String where) {
        return getDao().countAll(where);
    }

    /**
     * @return
     * @see net.sf.sidaof.dao.Dao#findAll()
     */
    public List<E> findAll() {
        return getDao().findAll();
    }

    /**
     * @param startPosition
     * @param maxResult
     * @return
     * @see net.sf.sidaof.dao.Dao#findAll(int, int)
     */
    public List<E> findAll(int startPosition, int maxResult) {
        return getDao().findAll(startPosition, maxResult);
    }

    /**
     * @param id
     * @return
     * @see net.sf.sidaof.dao.Dao#findById(java.lang.Object)
     */
    public E findById(K id) {
        return getDao().findById(id);
    }

    /**
     * 
     * @see net.sf.sidaof.dao.Dao#flush()
     */
    public void flush() {
        getDao().flush();
    }

    /**
     * @param entity
     * @return
     * @see net.sf.sidaof.dao.Dao#merge(java.lang.Object)
     */
    public E merge(E entity) {
        return getDao().merge(entity);
    }

    /**
     * @param entities
     * @see net.sf.sidaof.dao.Dao#persist(java.util.Collection)
     */
    public void persist(Collection<E> entities) {
        getDao().persist(entities);
    }

    /**
     * @param entity
     * @see net.sf.sidaof.dao.Dao#persist(java.lang.Object)
     */
    public void persist(E entity) {
        getDao().persist(entity);
    }

    /**
     * @param entity
     * @see net.sf.sidaof.dao.Dao#refresh(java.lang.Object)
     */
    public void refresh(E entity) {
        getDao().refresh(entity);
    }

    /**
     * @param entity
     * @see net.sf.sidaof.dao.Dao#remove(java.lang.Object)
     */
    public void remove(E entity) {
        getDao().remove(entity);
    }

    /**
     * Delegate execution of this class' methods to this DAO.
     * 
     * @return The DAO to delegate execution of this class' methods.
     */
    public abstract Dao<K, E> getDao();
}
